﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Sailboat.DrawingObjects;
using Sailboat.MovementStrategy;

namespace Sailboat.Generics
{
    internal class BoatsGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>> _boatStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _boatStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public BoatsGenericStorage(int pictureWidth, int pictureHeight)
        {
            _boatStorages = new Dictionary<string,
            BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (_boatStorages.ContainsKey(name))
            {
                return;
            }
            _boatStorages[name] = new BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (!_boatStorages.ContainsKey(name))
            {
                return;
            }
            _boatStorages.Remove(name);
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>?
        this[string ind]
        {
            get
            {
                if (_boatStorages.ContainsKey(ind))
                {
                    return _boatStorages[ind];
                }
                return null;
            }
        }
    }
}
