using Sailboat.Entities;
using Sailboat.DrawingObjects;
using Sailboat.MovementStrategy;

namespace Sailboat
{
    public partial class FormSailboat : Form
    {
        private DrawingBoat? _drawingBoat;
        private AbstractStrategy? _abstractStrategy;
        public DrawingBoat? SelectedBoat { get; private set; }
        public FormSailboat()
        {
            InitializeComponent();
            _abstractStrategy = null;
            SelectedBoat = null;
        }
        private void Draw()
        {
            if (_drawingBoat == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxSailboat.Width,
            pictureBoxSailboat.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingBoat.DrawTransport(gr);
            pictureBoxSailboat.Image = bmp;
        }
        private void buttonCreateBoat_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawingBoat = new DrawingBoat(random.Next(100, 300), random.Next(1000, 3000), color, pictureBoxSailboat.Width, pictureBoxSailboat.Height);
            _drawingBoat.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        private void buttonCreateSailboat_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }

            Color dopColor = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialog.Color;
            }

            _drawingBoat = new DrawingSailboat(random.Next(100, 300),
            random.Next(1000, 3000), color, dopColor, Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)),
            pictureBoxSailboat.Width, pictureBoxSailboat.Height);
            _drawingBoat.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawingBoat == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawingBoat.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawingBoat.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawingBoat.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawingBoat.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }
        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawingBoat == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(_drawingBoat.GetMoveableObject, pictureBoxSailboat.Width,
                pictureBoxSailboat.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            comboBoxStrategy.Enabled = false;
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }

        private void buttonSelectBoat_Click(object sender, EventArgs e)
        {
            SelectedBoat = _drawingBoat;
            DialogResult = DialogResult.OK;
        }
    }
} 